<?php
/**
 * データアクセスオブジェクト(雛形)
 *
 * @package Lambda/lib/DB/DAO
 * @author  rooth
 * @version 0.0.1
 *
 * PHP versions 5
 */

/**
 *<pre>
 *
 * このファイルは、DAOクラスの雛形です。
 * 
 * クラス名:   DAO_daoname
 * プロパティ: $table = 'table_name';
 * プロパティ: $pkey  = 'primary_key';
 * 
 * 上記を変更して利用します。
 *
 *</pre>
 */

/**
 *<pre>
 *
 * 推奨される メソッド命名規則
 *
 * [fetchAll]
 *  fetchAll($offset, $limit)
 *
 * [fetch]
 *  fetch_BY_id($id)
 *  fetch_BY_id_AND_name($id, $name)
 *  fetch_BY_id_OR_name($id, $name)
 *
 * [fetchColumn]
 *  fetchColumn_address_BY_id($id)
 *  fetchColumn_address_BY_id_AND_name($id, $name)
 *  fetchColumn_address_BY_id_OR_name($id, $name)
 *
 * [insert]
 *  insert()
 *
 * [update]
 *  update_name_address_BY_id($id)
 *  update_BY_id_AND_name($id, $name)
 *
 * [delete]
 *  delete_BY_id($id)
 *</pre>
 */
require_once dirname(__FILE__).'/DAO_Base.php';

class DAO_daoname extends DAO_Base
{
    protected $table = 'table_name';
    protected $pkey  = 'primary_key';

    public function __construct()
    {
        parent::__construct();
        $this->manipulator->setTable($this->table);
    }

    /**
     * よく利用される下記の参照系メソッドは、親クラス(DAO_Base)に実装されています。
     * 
     *     fetchAll()
     *     fetchAll_BY_ids()
     *     fetch_BY_id()
     * 
     * 親クラス(DAO_Base)に実装されていないメソッドは、以下の example のように実装します。
     */

    /**
     * example
     */
    public function fetchAll_BY_city($city, $offset = '', $limit = '')
    {
        $this->manipulator->select('SQL_CALC_FOUND_ROWS * ');
        $this->manipulator->where(array(
            'city' => $city,
        ));
        $this->manipulator->orderBy(array(
            $this->pkey => 'DESC',
        ));
        if ($offset !== '' && $limit !== '') {
            $this->manipulator->limit($offset, $limit);
        }
        return $this->manipulator->fetchAll();
    }

    /**
     * example
     */
    public function fetchAll_BY_ids(Array $ids)
    {
        $this->manipulator->select('SQL_CALC_FOUND_ROWS * ');
        $this->manipulator->where(array(
            $this->pkey => array('IN', $ids)
        ));
        return $this->manipulator->fetchAll();
    }

    /**
     * example
     */
    public function fetch_BY_name($name)
    {
        $this->manipulator->select('*');
        $this->manipulator->where(array(
            'name' => $name,
        ));
        return $this->manipulator->fetch();
    }



    /**
     * example: insert
     */
    public function insert(Array $p)
    {
        $this->manipulator->columns(
            'name', 'email', 'city', 'reg_datetime', 'upd_datetime'
        );
        $this->manipulator->values(array(
            'name'         => $p['name'], 
            'email'        => $p['email'], 
            'city'         => $p['city'], 
            'reg_datetime' => date('Y-m-d H:i:s'), 
            'upd_datetime' => NULL, 
        ));
       return $this->manipulator->insert();
    }

    /**
     * example: update
     */
    public function update($id, Array $p)
    {
        $this->manipulator->set(array(
            'name'         => $p['name'], 
            'email'        => $p['email'], 
            'city'         => $p['city'], 
            'upd_datetime' => date('Y-m-d H:i:s'), 
        ));
        $this->manipulator->where(array(
            $this->pkey => $id,
        ));
        return $this->manipulator->update();
    }

    /**
     * よく利用される下記の削除系メソッドは、親クラス(DAO_Base)に実装されています。
     * 
     *     delete_BY_id()
     *     delete_BY_ids()
     */



} //-- End of class


