<?php
/**
 * ダウンロード class
 *
 * @package Lambda/lib
 * @author  rooth
 * @version 0.0.1
 *
 * PHP version 5
 *
 *<pre>
 *
 * usage:
 *  require_once '/path/to/Download.php';
 *  require_once '/path/to/Mime.php';
 *
 *  // download and count
 *  $fpath = '/path/to/hoge.zip';
 *  $fname = 'hoge.zip';
 *  $dl = new Download($fpath, $fname);
 *  $dl->increment($counter_file);
 *  $dl->execute();
 *
 *  // get count
 *  $counter_file = '/path/to/counter/hoge.txt';
 *  echo Download::getCount($counter_file);
 *
 *</pre>
 */

class Download
{
    protected $fpath;
    protected $fname;

    function __construct($fpath, $fname = '')
    {
        $this->fpath = $fpath;
        $this->fname = ($fname === '') ? $this->getFilename($fpath) : $fname;
    }

    public function increment($counter_file)
    {
        if (is_readable($counter_file) && ! is_writable($counter_file)) {
            // read ok, write ng.
            exit("Fatal: Download: permission deny `$counter_file'.\n");
        } elseif ( ! is_writable($counter_file)) {
            // first write.
            if ( ! @error_log(1, 3, $counter_file)) {
                // write ng
                exit("Fatal: Download: permission deny `$counter_file'.\n");
            }
        } else {
            // increment
            $fp = fopen($counter_file, 'r+');
            flock($fp, LOCK_EX);
            $count = fread($fp, 1024);
            $count = (int)$count + 1;
            rewind($fp);
            fwrite($fp, $count);
            flock($fp, LOCK_UN);
            fclose($fp);
        }
    }

    public function execute()
    {
        $mimetype = Mime::getType($this->getExt($this->fname));
        header('Content-Type: '.$mimetype);
        header('Content-Length: '.filesize($this->fpath));
        header('Content-Disposition: attachment; filename="'.$this->fname.'"');
        echo file_get_contents($this->fpath);
        exit;
    }



    public static function getCount($counter_file)
    {
        return file_get_contents($counter_file);
    }



    protected function getFilename($fpath)
    {
        return array_pop(explode('/', $fpath));
    }

    protected function getExt($fname)
    {
        if (strpos($fname, '.') === false) return '';
        return array_pop(explode('.', $fname));
    }



}


