<?php
/**
 * Mime class
 *
 * @package Lambda/lib
 * @author  rooth
 * @version 0.0.1
 *
 * PHP version 5
 */

class Mime
{
    protected static $map = array(
        'txt'   => 'text/plain',
        'csv'   => 'text/csv',
        'xml'   => 'text/xml',
        'html'  => 'text/html',
        'htm'   => 'text/html',
        'gif'   => 'image/gif',
        'jpg'   => 'image/jpeg',
        'jpg'   => 'image/pjpeg',
        'jpeg'  => 'image/jpeg',
        'png'   => 'image/png',
        'png'   => 'image/x-png',
        'tif'   => 'image/tiff',
        'tiff'  => 'image/tiff',
        'bmp'   => 'image/x-bmp',
        'ps'    => 'appilcation/postscript',
        'eps'   => 'appilcation/postscript',
        'epsf'  => 'appilcation/postscript',
        'ai'    => 'application/postscript',
        'zip'   => 'application/zip',
        'lzh'   => 'application/x-lzh',
        'lha'   => 'application/octet-stream',
        'tar'   => 'application/x-tar',
        'gzip'  => 'application/x-tar',
        'cpt'   => 'application/mac-compactpro',
        'doc'   => 'application/msword',
        'docx'  => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
        'xls'   => 'application/vnd.ms-excel',
        'xlsx'  => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        'ppt'   => 'application/vnd.ms-powerpoint',
        'pptx'  => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
        'rtf'   => 'application/rtf',
        'pdf'   => 'application/pdf',
        'css'   => 'application/css',
        'au'    => 'audio/basic',
        'rpm'   => 'audio/x-pn-realaudio-plugin',
        'swa'   => 'application/x-director',
        'mp3'   => 'audio/mpeg',
        'mp4'   => 'audio/mp4',
        'wav'   => 'audio/x-wav',
        'midi'  => 'audio/midi',
        'avi'   => 'vide/x-msvideo',
        'mpeg'  => 'video/mpeg',
        'mpg'   => 'video/mpeg',
        'wmv'   => 'video/x-ms-wmv',
        'flash' => 'application/x-shockwave-flash',
        'exe'   => 'application/octet-stream',
        'hqx'   => 'application/mac-binhex40',
        'cpt'   => 'application/mac-compactpro',
        'php'   => 'application/x-httpd-php',
        'php4'  => 'application/x-httpd-php',
        'php3'  => 'application/x-httpd-php',
        'phtml' => 'application/x-httpd-php',
        'phps'  => 'application/x-httpd-php-source',
        'js'    => 'application/x-javascript',
        'swf'   => 'application/x-shockwave-flash',
        'eml'   => 'message/rfc822',
        '3gpp'  => 'video/3gpp',                        // docomo chaku-uta,movie
        '3gp'   => 'video/3gpp',                        // docomo chaku-uta,movie
        'amr'   => 'video/3gpp',                        // docomo chaku-uta,movie
        'asf'   => 'video/3gpp',                        // docomo chaku-uta,movie
        'jam'   => 'application/x-jam',                 // docomo i-appli
        'dmt'   => 'application/x-decomail-template',   // docomo decoration mail template
        'hdml'  => 'text/x-hdml',                       // au, docomo HDML
        '3gpp2' => 'video/3gpp2',                       // au chaku-uta,ez-movie
        '3g2'   => 'video/3gpp2',                       // au chaku-uta,ez-movie
        'amc'   => 'video/3gpp2',                       // au chaku-uta,ez-movie
        'kjx'   => 'application/x-kjx',                 // au ez-appli
        'khm'   => 'application/x-kddi-htmlmail',       // au decoration mail template
        'mmf'   => 'application/x-smaf ',               // softbank chakumero
        'smaf'  => 'application/x-smaf',                // softbank chakumero
        'hmt'   => 'application/x-htmlmail-template',   // softbank decoration mail template
        'jar'   => 'application/java-archive',          // java appli
        'jad'   => 'text/vnd.sun.j2me.app-descriptor',  // java appli
    );

    /**
     * マイムタイプ 返却
     * @param  string $extension 拡張子
     * @return string マイムタイプ
     * @access public
     * @static
     */
    public static function getType($extension)
    {
        $extension = preg_replace('/^\.+/', '', strtolower($extension));
        foreach (array_keys(self::$map) as $e) {
            if ($e === $extension) return self::$map[$e];
        }
        return 'application/octet-stream';
    }

    /**
     * 拡張子 返却
     * @param  string $mimetype マイムタイプ
     * @param  string $fname    ファイル名 ※マイムタイプから検出できなかった時、ファイル名の拡張子を返却
     * @return string 拡張子
     * @access public
     * @static
     */
    public static function getExtension($mimetype, $fname = '')
    {
        $mimetype = strtolower($mimetype);
        foreach (self::$map as $k => $v) {
            if ($v === $mimetype) return $k;
        }
        if ($fname !== '' && strpos($fname, '.') !== false) {
            $arr = explode('.', $fname);
            return array_pop($arr);
        }

        return '';
    }



}


