<?php
/**
 * アップロード class
 *
 * @package Lambda/lib
 * @author  rooth
 * @version 0.0.1
 *
 * PHP version 5
 *
 *<pre>
 *
 * usage:
 *  
 *  [html]
 *   <!--
 *   <form method="post" action="..." enctype="multipart/form-data"> // enctype の書き忘れに注意。
 *   <input type="file" name="upload_document[1]" />
 *   <input type="file" name="upload_image[1]" />
 *   <input type="file" name="upload_image[2]" />
 *   <input type="file" name="upload_image[3]" />
 *   -->
 *
 *   <!-- for PHPDocumentor -->
 *   &lt;form method="post" action="..." enctype="multipart/form-data"&gt; // enctype の書き忘れに注意。
 *   &lt;input type="file" name="upload_document[1]" /&gt;
 *   &lt;input type="file" name="upload_image[1]" /&gt;
 *   &lt;input type="file" name="upload_image[2]" /&gt;
 *   &lt;input type="file" name="upload_image[3]" /&gt;
 *
 *  [php]
 *   require_once '/path/to/Upload.php';
 *   
 *   $params = array();
 *   $params['upload_document'][1] = array('ワードなどの原稿', 2, 'MB');
 *   $params['upload_image'][1]    = array('画像1', 100, 'KB', '/^(gif|png|jpe?g)$/i');
 *   $params['upload_image'][2]    = array('画像2', 1, 'MB', '/^(gif|png|jpe?g)$/i');
 *   $params['upload_image'][3]    = array('画像3', 1, 'MB', '/^(gif|png|jpe?g)$/i');
 *   
 *   $upload = new Upload($params);
 *   
 *   // アップロード関連の設定を表示
 *   $upload->getInfo();
 *   
 *   $upload->setUploadDir('/path/to/upload');
 *   
 *   // (optional) yyyymmdd形式のサブディレクトリを生成する
 *   $upload->setUploadDir('/path/to/upload')->subDir('%yyyymmdd%');
 *   
 *   // (optional) 名前を指定してサブディレクトリを生成する
 *   $upload->setUploadDir('/path/to/upload')->subDir('subdirname');
 *   
 *   if ($upload->execute() === false) {
 *       echo $upload->getErrMsg();
 *   } else {
 *       var_dump($upload->getResults());
 *   }
 *   
 *   array(1) {
 *     [0]=>
 *     array(3) {
 *       ["fieldlabel"]=>
 *       string(13) "画像1"
 *       ["orig_filename"]=>
 *       string(49) "画像1.jpg"
 *       ["filename"]=>
 *       string(35) "20090406_123527_upload_image_1_1214.jpg"
 *       ["filepath"]=>
 *       string(90) "/path/to/upload/20090406_123527_upload_image_1_1214.jpg"
 *     }
 *     [1]=>
 *     array(3) {
 *     }
 *     :
 *     :
 *   }
 *
 *</pre>
 */

/*
エラーメッセージ

UPLOAD_ERR_OK
    値: 0; エラーはなく、ファイルアップロードは成功しています。

UPLOAD_ERR_INI_SIZE
    値: 1; アップロードされたファイルは、php.ini の upload_max_filesize ディレクティブの値を超えています。

UPLOAD_ERR_FORM_SIZE
    値: 2; アップロードされたファイルは、HTML フォームで指定された MAX_FILE_SIZE を超えています。

UPLOAD_ERR_PARTIAL
    値: 3; アップロードされたファイルは一部のみしかアップロードされていません。

UPLOAD_ERR_NO_FILE
    値: 4; ファイルはアップロードされませんでした。

UPLOAD_ERR_NO_TMP_DIR
    値: 6; テンポラリフォルダがありません。PHP 4.3.10 と PHP 5.0.3 で導入されました。

UPLOAD_ERR_CANT_WRITE
    値: 7; ディスクへの書き込みに失敗しました。PHP 5.1.0 で導入されました。

UPLOAD_ERR_EXTENSION
    値: 8; ファイルのアップロードが拡張モジュールによって停止されました。 PHP 5.2.0 で導入されました。
*/

defined('DS') || define('DS', DIRECTORY_SEPARATOR);

class Upload
{
    protected $params     = array();
    protected $upload_dir = '';
    protected $sub_dir    = '';

    protected $err_msg = '';
    protected $results = array();

    /**
     * phpアップロード：エラーメッセージの説明
     * @see http://jp.php.net/manual/ja/features.file-upload.errors.php
     */
    const SUCCESS       =   1;
    const NO_FILE       =   2;
    const ERR_FILE_SIZE =  11;
    const ERR_FILE_TYPE =  12;
    const ERR_MOVE      =  13;
    const ERR_INI_SIZE  = 101;
    const ERR_FORM_SIZE = 102;
    const ERR_FAILURE   = 999;

    public function __construct(array $params)
    {
        $this->params = $params;
    }

    public function setUploadDir($upload_dir)
    {
        is_writable($upload_dir) || exit("Upload: Fatal: Directory not found (or permission denied): $upload_dir");

        $this->upload_dir = $upload_dir;
        return $this;
    }

    public function subDir($dirname)
    {
        $this->sub_dir = $dirname;
    }



    public function getInfo()
    {
        echo '<pre>';
        echo 'file_uploads = '.ini_get('file_uploads')." (HTTPファイルアップロードを有効とするかどうか)\n";
        echo 'max_input_time = '.ini_get('max_input_time')." (POST、GET、ファイルアップロードなどの入力をパースする最大時間)\n";
        echo 'upload_max_filesize = '.ini_get('upload_max_filesize')." (アップロードされるファイルの最大サイズ)\n";
        echo 'memory_limit = '.ini_get('memory_limit')." (スクリプトが確保できる最大メモリをバイト数)\n";
        echo 'post_max_size = '.ini_get('post_max_size')." (POSTデータに許可される最大サイズ)\n";
        echo 'max_execution_time = '.ini_get('max_execution_time')." (スクリプトが強制終了されるまでに許容される最大時間)\n";
        exit;
    }



    /**
     * @return 成功: array array(array(fieldlabel, orig_filename, filename, filepath), ...)
     *         失敗: bool false
     * @access public
     */
    public function execute()
    {
        $stack = array();
        foreach ($this->params as $key => $val) {
            foreach ($val as $idx => $param) {
                list($status, $fieldlabel, $orig_filename, $filename, $filepath) = $this->save($key, $idx, $param);

                //if ($status === self::NO_FILE) continue;

                $stack[] = array($fieldlabel, $orig_filename, $filename, $filepath);

                if ($status === self::SUCCESS || $status === self::NO_FILE) {
                    $this->setResult($fieldlabel, $orig_filename, $filename, $filepath);
                    continue;
                } else {
                    $this->remove($stack);
                    $this->setErrMsg($status, $param);
                    return false;
                }
            }
        }
        return $stack;
    }

    /**
     * @param  array array(ファイル名, 配列キー, array(フィールドラベル, 最大ファイルサイズ, 許可する拡張子の正規表現))
     * @return array array(status, フィールドラベル, オリジナルファイル名, ファイル名, ファイルパス)
     * @access protected
     */
    protected function save($key, $idx, array $param)
    {
        // 初期化
        $f_name     = isset($_FILES[$key]['name'][$idx]) ? $_FILES[$key]['name'][$idx] : '';
        $f_type     = isset($_FILES[$key]['type'][$idx]) ? $_FILES[$key]['type'][$idx] : '';
        $f_tmp_name = isset($_FILES[$key]['tmp_name'][$idx]) ? $_FILES[$key]['tmp_name'][$idx] : '';
        $f_error    = isset($_FILES[$key]['error'][$idx]) ? $_FILES[$key]['error'][$idx] : '';
        $f_size     = isset($_FILES[$key]['size'][$idx]) ? $_FILES[$key]['size'][$idx] : 0;

        // debug.
        // <form method="post" action="..." enctype="multipart/form-data"> // enctype の書き忘れに注意。
        //var_dump($f_error);
        //var_dump($_FILES);exit;

        // HTTP POST によりアップロードされたファイルでない
        if ($f_tmp_name !== '' && ! is_uploaded_file($f_tmp_name)) exit('Upload: Fatal: invalid request.');

        if ($f_name === '') {
            // FILE feild が空
            return array(self::NO_FILE, $param[0], '', '', '');
        }

        if ($f_size > ($param[1] * self::getBytes($param[2]))) {
            // ファイルサイズ超過
            return array(self::ERR_FILE_SIZE, $param[0], $f_name, '', '');
        }

        if (isset($param[3]) && ! preg_match($param[3], self::getExtension($f_name))) {
            // FILE TYPE 不一致
            return array(self::ERR_FILE_TYPE, $param[0], $f_name, '', '');
        }

        if ($f_error === UPLOAD_ERR_OK) {
            // アップロード成功

            if ($this->sub_dir !== '') {

                if ($this->sub_dir === '%yyyymmdd%') {
                    /* yyyymmddディレクトリの subdir バージョン */
                    $this->sub_dir = date('Ymd');
                }

                if ( ! is_writable($this->upload_dir.DS.$this->sub_dir)) {
                    $umask = umask(0);
                    mkdir($this->upload_dir.'/'.$this->sub_dir, 0777);
                    umask($umask);
                }
                $filename = $this->sub_dir.DS.date('His').'_'.$key.'_'.$idx.'_'.getmypid().self::getExtension($f_name, true);
                $filepath = $this->upload_dir.DS.$filename;
            } else {
                /* subdir を作成しないバージョン */
                $filename = date('Ymd_His').'_'.$key.'_'.$idx.'_'.getmypid().self::getExtension($f_name, true);
                $filepath = $this->upload_dir.DS.$filename;
            }



            if (move_uploaded_file($f_tmp_name, $filepath)) {
                // 移動成功
                $umask = umask(0);
                chmod($filepath, 0777);
                umask($umask);
                return array(self::SUCCESS, $param[0], $f_name, $filename, $filepath);
            } else {
                // 移動失敗
                return array(self::ERR_MOVE, $param[0], $f_name, '', '');
            }
        } elseif ($f_error === UPLOAD_ERR_INI_SIZE) {
            // php.ini の upload_max_filesize  ディレクティブの値を超えた
            return array(self::ERR_INI_SIZE, $param[0], $_FILES[$key]['name'][$idx], '', '');
        } elseif ($f_error === UPLOAD_ERR_FORM_SIZE) {
            // HTML フォームで指定された MAX_FILE_SIZE を超えた
            return array(self::ERR_FORM_SIZE, $param[0], $f_name, '', '');
        } else {
            // アップロード失敗
            return array(self::ERR_FAILURE, $param[0], $f_name, '', '');
        }
    }

    /**
     * @access public
     */
    public function getErrMsg()
    {
        return $this->err_msg;
    }

    /**
     * @access public
     */
    public function getResults()
    {
        return $this->results;
    }


    /**
     * @param array array(fieldlabel, orig_filename, filename, filepath);
     * @access protected
     */
    protected function remove(array $arr)
    {
        foreach ($arr as $v) if ($v[3] !== '') unlink($v[3]);
    }

    /**
     * @access protected
     */
    protected function setResult($fieldlabel, $orig_filename, $filename, $filepath)
    {
        $this->results[] = array(
            'fieldlabel'    => $fieldlabel, 
            'orig_filename' => $orig_filename, 
            'filename'      => $filename, 
            'filepath'      => $filepath,
        );
    }

    /**
     * 拡張子 返却
     * @param  string $fname ファイル名
     * @param  boot $append_dot ドットを付けるか返却するか否か
     * @return string 拡張子
     * @access public
     * @static
     */
    protected static function getExtension($fname, $append_dot = false)
    {
        if (strpos($fname, '.') !== false) {
            $arr = explode('.', $fname);
            if ($append_dot) {
                return '.' . strtolower(array_pop($arr));
            } else {
                return strtolower(array_pop($arr));
            }
        }

        return '';
    }

    /**
     * @param string $unit 単位 {KB|MB}
     * @return integer バイト数
     * @access protected
     */
    protected static function getBytes($unit)
    {
        $bytes = 1;
        if (strtoupper($unit) === 'KB') $bytes = 1024;
        if (strtoupper($unit) === 'MB') $bytes = 1024 * 1024;
        return $bytes;
    }



    /**
     * @param int $status ステータス
     * @param array $param array(フィールドラベル, ファイル名, ファイルパス)
     * @access public
     */
    public function setErrMsg($status, array $param)
    {
        switch ($status) {
            case self::ERR_FILE_SIZE:
                $this->err_msg  = 
                    '「'.$param[0].'」のファイルは '.$param[1].' '.$param[2].' を超えているため、アップロードできません。';
                break;

            case self::ERR_FILE_TYPE:
                $this->err_msg = '「'.$param[0].'」のファイルは形式が不正のため、アップロードできません。';
                break;

            case self::ERR_MOVE:
            case self::ERR_INI_SIZE:
            case self::ERR_FORM_SIZE:
            case self::ERR_FAILURE:
            default:
                $this->err_msg = '「'.$param[0].'」のアップロードに失敗しました(エラーコード:'.$status.')';
                break;
        }
    }



} // -- End of class


